#**********************************************************
# Script : omslewchain - 1.8.2000 Version
#
# Authors:  Jon Rainger - MSSL
#           Chris Brindle - CB
# Description: Run the OM slew flatfield pipeline.
#
# Comments:
#
# $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
#
#***********************************************************

goto ten;
    ten;
use SAS;
use DAL;

sub omslewchain {
#/////////////////////////////////////////
# AMK - removed the die statments in order to get the system to run
# AMK - dump messages to file
#    open (STDOUT, ">omslewchain_log");
#    open (STDERR, ">&STDOUT");
#    S| = 1;
#    select (STDERR);
#    S| = 1;
#////////////////////////////////////////    


#**************************************************
#
# The next few lines are copied from OMICHAIN
#
#*************************************************
   
    $taskname = 'OMSLEWCHAIN';
    $VERSION = '1.3';
    $date = &date_time;
    $create = "Running SAS task $taskname V$VERSION $date";

    $oal_directory = stringParameter("oaldirectory");
    if ($oal_directory eq "")
    {
        $oal_directory = $ENV{SAS_ODF};
    }
    $inp_directory = stringParameter("inpdirectory");
    if ($inp_directory eq "")
    {
        $inp_directory = $oal_directory;
    }
    $out_directory = stringParameter("outdirectory");
    if ($out_directory eq "")
    {
        $out_directory = $inp_directory;
    }
  SAS::message($create);
#    print("$create \n");
    print("oaldirectory = $oal_directory \n");
    print("inpdirectory = $inp_directory \n");
    print("outdirectory = $out_directory \n");
    print("Environment = $ENV{SAS_ODF} \n");


# Get and create the archive root directory 
$arch = stringParameter("archiveroot");

  if ( $arch  eq "") {
      $arch = $ENV{ARCH_ROOT};
  }

  if ( $arch  eq "") {
      $arch = "./archive";
  }

  if ( !( -e $arch )) {
  
      system("mkdir $arch") && die("Cannot create directory $arch\n");
   }

print("arch root = $arch\n");  

#***********************************************************
# Get a list of file names in the current working directory.
#***********************************************************

@filenames = <*> ;   
print("No of files in directory = $#filenames \n");

#******************************************************************
# $numberMod8Files has been added by CB (21.7.2000) to record the 
# of mod8 files that have been created
#******************************************************************
$numberMod8Files = 0;      

$IMIFiles = 0;
$NPHFiles = 0;
$PEHFiles = 0;
 
$NoExposures = 0;
$OldExposure = "";
@ExposureList = ();     # A list of the exposures in the data set.
@ImageFileList = ();    # Added by CB for test purposes

#***********************************************
# Check to see that there is an odf summary file
#***********************************************   
if ( &FindFile("ODF_SUM.SAS") eq 0)
{
    print("ODF_SUM.SAS FILE NOT FOUND!");
}
#****************************************************
$string="$inp_directory/ODF_SUM.SAS";
#print("$string \n");
#print("Input directory=$inp_directory\n"); 
system("setenv SAS_ODF $string");


#******************************************************
# Check for the presence of an index file
#
# Create one if necessary
#*****************************************
if ( &FindFile("INDEX.FIT") eq 0)
{
    print("... omflatindex - creating INDEX file \n");

    $arg_list = "operation=create" .
	        " indexset=INDEX.FIT";
    system("omflatindex $arg_list") && die("omflatindex failed\n");
#    print ("errno= $! \n");
#print("$@ \n");    

#$exit_value = $?>>8;
#print("exitvalue=$exit_value\n");
#$signal_num = $? &127;
#print("signalnum=$signal_num\n");
#print("exitvalue=$exit_value\n");        
}

#$num = 1;
#    for ($i=1; $i < 100; $i++)
#    {
#        $num = $num + 1;
#        if ($num eq 10) {goto escape;}
#    }
#escape:
#    print("number = $num \n");



# die;

#******************************************************************************
# Sort the file names according to observation, exposure, window and file type.
#******************************************************************************
#system("rm *_new");   
foreach $file (@filenames)
{
   if ($file =~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM[S|U|X]([0-9a-zA-Z]{3})([0-9]{2})([a-zA-Z]{3}).FIT/i)
   {

       
      if ("$3" ne $OldExposure & "$3" ne "000")
      {
          $NoExposures = $NoExposures + 1;
          $OldExposure = "$3";
          push(@ExposureList, "$3");
          print("Exposure $NoExposures = $3 \n");
           
      } 

      if ($file =~ /NPH/i)
      {
	  $NPHFiles = $NPHFiles + 1;
      }    
      elsif ($file =~ /PEH/i)
      {
	 $PEHFiles = $PEHFiles + 1;
      } 
      else #if ($file =~ /IMI/i)
      {
 
      
#	 if ($file =~ /IMI/i) 
         {
#             print ("IMI file= $file \n");   
             $IMIFiles = $IMIFiles + 1;
             push(@ImageFileList, "$file");
    
         }
      

         # Orbit number list
         $orb_list{$1} = "$1";
         
         # Observation list.
         ${${"$1"}{$2}} = "$1$2";
#         print("observation list = ${${"$1"}{$2}} \n");
       
         # Exposure list.
         ${${"$1$2"}{$3}} = "$1$2$3";
    
         # Window list. 
         ${${"$1$2$3"}{$4}} = "$1$2$3$4";
     
         # File list.
         ${${"$1$2$3$4"}{$5}} = "$file";
     }
   }
}
print("No of exposures = $NoExposures \n");
foreach $item (@ExposureList)
{
    print("Exposure - $item \n");
}


#foreach $item (@ImageFileList)
#{
#    print("Image file - $item \n");
#}


print("No of NPH files = $NPHFiles \n");
print("No of PEH files = $PEHFiles \n");
print("No of IMI files = $IMIFiles \n");

if ($NPHFiles eq 0)
{
    die("No NPH file \n");
}
if ($PEHFiles eq 0)
{
    die("No PEH file \n");
}

if ($IMIFiles eq 0)
{
    die("No IMI file \n");
}

#******************************************************
# Now run the pipeline- Loop over all observations.
#******************************************************
foreach $orb_key (keys(%orb_list))
{
    @list = %orb_list;
   
    $obs_list = $orb_list{$orb_key};
    
    print("processing Orbit $orb_key \n");
  
    foreach $obs_key (keys(%{$obs_list}))
    {
       @list = (%{$obs_list});
       $number = $#list + 1;
      
    
       # Loop over all exposures.
       $exp_list = ${${$obs_list}{$obs_key}};  
       @raw_sdf_image_list = ();
       @combined_image_list = ();
       @mod8combined_image_list = ();
    
       foreach $exp_key (keys(%{$exp_list}))
       {
           @list = (%{$exp_list});
           $number = $#list + 1;
             
         
           #*****************************************************
           # Check to see if Window Data File is present
           # and periodic and non-periodic house-keeping 
           # files (Added by CB - 5.6.2000).
           #*****************************************************     
           if ($window_data_filename = $orb_key . "_" . $obs_key . "_OMS" . $exp_key . "00WDX.FIT")
           {
               $window_file_flag = 1;
               print("window file name = $window_data_filename \n");
           }
           else
           {
               kill("Window file doesn't exist! \n");
           }      
     
           # There is now only one PEH file
           if ($periodic_hk_filename = $orb_key . "_" . $obs_key . "_OMX00000PEH.FIT")
           {
               $periodic_hk_file_flag = 1; 
               print("periodic hk file = $periodic_hk_filename \n");
           }        
           else
           {
               kill("Periodic file doesn't exist! \n");
           }      
     
           # There is now only one NPH file              
           if ($non_periodic_hk_filename = $orb_key . "_" . $obs_key . "_OMX00000NPH.FIT")
           {
               $non_periodic_hk_file_flag = 1;
               print("non_periodic file name = $non_periodic_hk_filename \n");
           }
           else
           {
               kill("Non-periodic file doesn't exist! \n");
           }      
           print("Setting pipeline variables\n");

           # Set pipeline variables.
	   $win_list = ${${$exp_list}{$exp_key}};
#           print("win_list=$win_list\n"); 
           @sdf_exposure_image_list = ();       
           @raw_exposure_image_list = ();

                      
           foreach $win_key (keys(%{$win_list}))
           {
               # Prepare images for pipeline
               $sdf_image_filename = $orb_key . "_" . $obs_key . "_OMS" . $exp_key . $win_key . "IMI.FIT";
               $image_filename = $orb_key . "_" . $obs_key . "_OMS" . $exp_key . $win_key . "IMI.FIT_new";
	       $raw_sdf_image_filename = "../../" . $orb_key . "_" . $obs_key . "_OMS" . $exp_key . $win_key . "IMI.FIT";
	       
               print("           Slew Observation $obs_key \n");

               #************************************************* 
               #   omslewchain
               # AMK - ammended to run omprep - 23.05.2000	    
               #*************************************************              
               print("... omprep...\n");
	       $arg_list = "set=" . $sdf_image_filename .
		           " wdxset=" . $window_data_filename .
			   " outset=" . $image_filename .
                           " pehset=" . $periodic_hk_filename .           
			   " nphset=" . $non_periodic_hk_filename .      
                           " modeset=2";   # This parameter informs omprep that the slewchain is being run.    

               print("omprep $arg_list\n");

               system("omprep $arg_list")  && die("omslewchain failed\n");

               push(@sdf_exposure_image_list, $sdf_image_filename);
               push(@raw_exposure_image_list,  $image_filename);
               push(@raw_sdf_image_list,  $raw_sdf_image_filename); 
             
           }   # End of "foreach $win_key (keys(%{$win_list}))"


           #***************************************** 
           # Insert mkdir for archive subdirectory
           #*****************************************
           if ( !( -e ($arch . "/" . $obs_key) ))
           {
               system("mkdir $arch/$obs_key") && die("cant make subdirectory $obs_key\n"); 
	   }

           $combined_filename =   "OM" . $exp_key . "00CO.FIT";
           $mod8combined_filename =   "OM" . $exp_key . "00M8.FIT";
           $modulo_8_product_filename =  "P" . $obs_key . "/OM" . $exp_key . "MOD8MP" . "00COMB" . "XXXV.FIT";

           #**************************************
           # Produce tracking history products.
           # omcomb.
           #**************************************
  
           print("... omcomb ...\n");

           $arg_list = " filelistset=\"@raw_exposure_image_list\"" .
	                " outset=" .  $combined_filename ;
           print("omcomb $arg_list\n");
           system("omcomb $arg_list")  && die("omcomb failed\n");

           #***************************************
           # Copy combined image to archive
           #***************************************
           system("mv  $combined_filename ./$arch/$obs_key/$combined_filename");

           #*****************************************************
           # omcosflag.
           #*****************************************************
           print("... omcosflag ...\n");
           $arg_list = "set=" . $arch . "/" . $obs_key . "/" . $combined_filename .  
                        " thxset=" . "tmp_tracking";
           print("omcosflag $arg_list\n");
           system("omcosflag $arg_list") && die("omcosflag failed\n");
           print("\n\nfinished omcosflag\n");

           #*******************************************
           # ommodmap.
           #*******************************************
           print("... ommodmap ...\n");
           $arg_list = "set=" . $arch . "/" . $obs_key . "/" . $combined_filename . 
                        " outset=" . $mod8combined_filename .
                        " mod8product=yes" .
                        " mod8set=" . $modulo_8_product_filename .
                          " mod8set=" . $inp_directory . '/' . $modulo_8_product_filename .                      
                        " nsig=5" .
                        " nbox=3";
           print("\n\nommodmap $arg_list\n");
           print("\n\nAbout to execute ommodmap\n");
           system("ommodmap $arg_list") && die("ommodmap failed\n");
           print("\n\nFINISHED ommodpmap");

           system("mv  $mod8combined_filename ./$arch/$obs_key/$mod8combined_filename");
           push(@combined_image_list, $combined_filename);
	   push(@mod8combined_image_list, $mod8combined_filename);
           $numberMod8Files = $numberMod8Files + 1;
                                 

           if ($numberMod8Files eq 4)
           {
      
               #**************************************************
               # omflatindex
               #**************************************************
               print("... omflatindex ...\n");

               $arg_list = "operation=insert" .
                     " indexset=INDEX.FIT" .
                     " mod8sets=\"@mod8combined_image_list\"" .
                     " combinedsets=\"@combined_image_list\"" .
                     " ppsversion=00001" .
                     " rootdirectory=$arch/$obs_key" ;
               print("omflatindex $arg_list\n");
               system("omflatindex $arg_list") && die("omflatindex failed\n");
#               system("rm *_new");
               @combined_image_list = ();           
               @mod8combined_image_list = ();
               $numberMod8Files = 0;
	   
           } # The end of "if ($numberMod8Files eq 4)"
     
   
       } # The end of "foreach $exp_key (keys(%{$exp_list}))"
#************************************* 

   } # End of "foreach $obs_key (keys(%{$obs_list}))"
}  # End of "foreach $orb_key (keys(%orb_list))

$date = &date_time;
$finished = "Finished running the slew chain - $date";

#print("\n\n$finished\n");


#////////////////////////////////////////////////
# AMk - close the log file
#   close(STDOUT);
#   close(STDERR);
#///////////////////////////////////////////////

}  # end of sub omslewchain 
#****************************************************
#
# date_time - Returns the date and time as a string
#             
#****************************************************
sub date_time
{
    local $dateString;

    use POSIX(strftime);
    $dateString = strftime "%a %b %e %H:%M:%S %Y", gmtime;
   
    return $dateString;
}
#*****************************************************
# FindFile - Searches the directory for the file $
#
#*****************************************************
sub FindFile($)
{
    my ( $fileName) = @_;
    local $number = 0;

    print("Searching for file $fileName\n");
    foreach $file (@filenames)
    {
        if ($file eq $fileName) 
        {
            $number = $number + 1;
        } 
    }
    if ($number eq 0)
    {
        print("File not found\n");
    }
    return $number;    

}

       
    
    





















